/*
 *  This program fixes a problem with PHost 4.0k:
 *
 *  If a universe is mastered in such a way that Master considers some
 *  planets non-existant, those will have an Id of zero. This causes a
 *  crash in 4.0k's experience processing.
 *
 *  This program will set these planets' Id numbers, and at the same
 *  time move the affected planets out of sight.
 *
 *  Instructions: if your host run crashes with
 *     You've encountered a bug in PHost. Please contact the PHost support group
 *     at <support@phost.de>. Please include the following information:
 *           File:          accessor.c         <- important part
 *           Line:          76                 <- important part
 *           PHost version: 4.0k (beta)
 *           Environment:   PC Linux
 *           Context:       Experience         <- important part
 *  run this program in your game directory (i.e. copy the program
 *  into the game directory, 'cd' in there, and run the program).
 *
 *  Your game should continue unchanged.
 */

#include <stdio.h>
#include <stdlib.h>

unsigned char pdata[42502];
unsigned char xyplan[3000];

void loadFile(unsigned char* buf, size_t size, const char* name)
{
    FILE* fp = fopen(name, "rb");
    if (!fp) {
        perror(name);
        exit(1);
    }
    if (fread(buf, 1, size, fp) != size) {
        fclose(fp);
        fprintf(stderr, "%s: file has invalid size\n", name);
        exit(1);
    }
    fclose(fp);
}

void saveFile(unsigned char* buf, size_t size, const char* name)
{
    FILE* fp = fopen(name, "wb");
    if (!fp) {
        perror(name);
        exit(1);
    }
    if (fwrite(buf, 1, size, fp) != size) {
        fclose(fp);
        fprintf(stderr, "%s: can't write\n", name);
        exit(1);
    }
    fclose(fp);
}

unsigned int get(const unsigned char* p)
{
    return p[0] + 256*p[1];
}

void put(unsigned char* p, unsigned int n)
{
    p[0] = n & 255;
    p[1] = (n >> 8) & 255;
}

int main(void)
{
    int pid;
    int changes;

    loadFile(pdata, sizeof(pdata), "pdata.hst");
    loadFile(xyplan, sizeof(xyplan), "xyplan.dat");

    changes = 0;
    for (pid = 1; pid <= 500; ++pid) {
        /* Planet record:
             +0   player Id
             +2   planet Id */
        unsigned char* planet = &pdata[2 + 85*(pid-1)];
        /* XY record:
             +0   X
             +2   Y */
        unsigned char* xyreg  = &xyplan[6 * (pid-1)];

        if (get(&planet[2]) == 0) {
            /* Planet has an Id number of zero. This causes PHost 4.0k
               to crash. We set it to nonzero, and move the planet out
               of the way. There is no Y checksum, so we can freely
               move that. */
            put(&planet[0], 0);
            put(&planet[2], pid);
            put(&xyreg[2], 9000);
            ++changes;
        }
    }

    if (changes == 0) {
        printf("Nothing changed. Universe seems intact.\n"
               "If PHost still crashes, you have a different bug.\n");
    } else {
        printf("Changed %d planet records\n", changes);
        saveFile(pdata, sizeof(pdata), "pdata.hst");
        saveFile(xyplan, sizeof(xyplan), "xyplan.dat");
    }
    return 0;
}
